' Intercogcall v00.00.01
'
' An experiment in dispatching functions accross COGs.
'
' Three COGs are running PASM code.
' The first one contains functions that are invoked from the main Spin code.
' The second and third COGs conatin functions that are invoked from the first.
'
' v00-00 2009-11-14          First version
'
' v00-01 2009-11-15          Use address of get_cmd as the null command instead of zero.
'

CON
   _clkmode = xtal1 + pll16x
   _xinfreq = 6_553_600

OBJ
  term : "FullDuplexSerial"

PUB start

  'Initialize counter
  count := 10

  term.start (31, 30, 0, 115200)
  term.tx(13)
  term.str(string("Inter COG call test", 13))

  displayResult

  'Start our first COG
  cognew (@enter_0, 0)

  'Start our second COG
  cognew (@enter_1, 0)

  'Start our third COG
  cognew (@enter_2, 0)

  term.str(string("Test 1", 13))
  'Invoke first COGS count_up function
  repeat 20
    countDown
  displayResult

  term.str(string("Test 2", 13))
  'Invoke first COGS count_down function
  repeat 15
    countUp
  displayResult

  term.str(string("Test 3", 13))
  'Invoke first COGS reset count function
  countReset
  displayResult

  'Hang up here with final result displayed.
  repeat

PRI countUp
  cmd_0 := count_up_cmd
  repeat while cmd_0 == count_up_cmd    'Wait for the command to complete

PRI countDown
  cmd_0 := count_down_cmd
  repeat while cmd_0 == count_down_cmd  'Wait for the command to complete

PRI countReset
  cmd_0 := count_reset_cmd
  repeat while cmd_0 == count_reset_cmd 'Wait for the command to complete

PRI displayResult
  term.str(string("Counter = "))
  term.dec(count)
  term.tx(13)
  term.str(string("Overflow = "))
  term.dec(overflow)
  term.tx(13)
  term.str(string("Underflow = "))
  term.dec(underflow)
  term.tx(13)

DAT
'---------------------------------------------------------------------------------------------------
'
                 org     0
enter_0          jmp     #get_cmd_0

done_0           wrlong  get_cmd_0, cmd_ptr_0   'Clear the given command, indicates done
get_cmd_0        rdlong  temp_0, cmd_ptr_0 wz   'Read a command from HUB
                 jmp     temp_0                 'Jump to the required function (no # here)

count_up         rdlong  temp_0, count_ptr      'Fetch the counter from HUB
                 add     temp_0, #1 wc          'Increment it
                 wrlong  temp_0, count_ptr      'And write it back

           if_nc jmp     #done_0                'Check for overflow

                 mov     temp_0, #on_overflow   'Invoke on_overflow in slave COG 1
                 wrlong  temp_0, cmd_ptr_1s     '
:wait            rdlong  temp_0, cmd_ptr_1s     'and wait for it to complete
                 cmp     temp_0, #on_overflow wz
           if_z  jmp     #:wait
                 jmp     #done_0

count_down       rdlong  temp_0, count_ptr      'Fetch the counter from HUB
                 sub     temp_0, #1 wc          'Decrement it
                 wrlong  temp_0, count_ptr      'And write it back
           if_nc jmp     #done_0                'Check for underflow

                 mov     temp_0, #on_underflow  'Invoke on_underflow in slave COG 2
                 wrlong  temp_0, cmd_ptr_2s
:wait            rdlong  temp_0, cmd_ptr_2s wz  'and wait for it to complete
                 cmp     temp_0, #on_underflow
           if_z  jmp     #:wait
                 jmp     #done_0

count_reset      mov     temp_0, #0             'Counter value zero
                 wrlong  temp_0, count_ptr      'And write it back
                 jmp     #done_0

cmd_ptr_0        long    @@@cmd_0               'Pointer to this COGs command in HUB.
cmd_ptr_1s       long    @@@cmd_1               'Pointer to slave COG 1 command in HUB.
cmd_ptr_2s       long    @@@cmd_2               'Pointer to slave COG 2 command in HUB.

count_ptr        long    @@@count               'Pointer to counter in HUB

temp_0           long    0

                 fit     $1f0

cmd_0            long   get_cmd_0               'Commands to COG are written here

count_up_cmd     long   count_up                'Commands implemented this COG
count_down_cmd   long   count_down
count_reset_cmd  long   count_reset

count            long   0                       'The counter value in HUB

DAT
'---------------------------------------------------------------------------------------------------
'
                 org    0

enter_1          jmp     #get_cmd_1

done_1           wrlong  get_cmd_1, cmd_ptr_1   'Clear the given command, indicates done
get_cmd_1        rdlong  temp_1, cmd_ptr_1 wz   'Read a command from HUB
                 jmp     temp_1                 'Jump to the required function (no # here)


on_overflow      mov     temp_1, #1             'Set overflow indicator
                 wrlong  temp_1, overflow_ptr
                 jmp     #done_1



cmd_ptr_1        long    @@@cmd_1               'Pointer to this COGs command in HUB.

overflow_ptr     long    @@@overflow            'Pointer to overflow indicator
temp_1           long    0

                 fit     $1f0

cmd_1            long     get_cmd_1             'Commands to COG are written here

overflow         long     0                     'The overflow indicator

DAT
'---------------------------------------------------------------------------------------------------
'
                 org     0

enter_2          jmp     #get_cmd_2

done_2           wrlong  get_cmd_2, cmd_ptr_2   'Clear the given command, indicates done
get_cmd_2        rdlong  temp_2, cmd_ptr_2 wz   'Read a command from HUB
                 jmp     temp_2                 'Jump to the required function (no # here)

on_underflow     mov     temp_2, #1             'Set underflow indicator
                 wrlong  temp_2, underflow_ptr
                 jmp     #done_2

cmd_ptr_2        long    @@@cmd_2               'Pointer to this COGs command in HUB.

underflow_ptr    long    @@@underflow           'Pointer to underflow indicator
temp_2           long    0

                 fit     $1f0

cmd_2            long    get_cmd_2             'Commands to COG are written here

underflow        long     0                     'The underflow indicator

